#!/usr/bin/env python3

"""Plots outermost contours and max-speed contours.

Red for anti-cyclones, blue for cyclones. Empty circles for extrema
with a valid outermost contour, empty squares for other
extrema. Squares on outermost contour for a well-defined but invalid
outermost contour. Crosses on outermost contour for a valid outermost
contour but with no distinct max-speed contour. Filled circles on
outermost contour and max-speed contour for a valid outermost contour
with a distinct max-speed contour.

This scripts takes about 30 s of CPU for a 90° by 90° window, about 4
mn for a 180° by 180° window.

"""

import argparse
import shapefile
import numpy as np
import matplotlib.pyplot as plt
import netCDF4
import math
import itertools
import cartopy.crs as ccrs
import sys

parser = argparse.ArgumentParser()
parser.add_argument("-v", "--velocity", help = "plot velocity field",
                    action = "store_true")
parser.add_argument("-s", "--scale", default = 20, type = float,
                    help = "scale of arrows for the velocity field")
parser.add_argument("-g", "--grid", help = "plot grid", action = "store_true")
parser.add_argument("-w", "--window", help = "choose a limited plot window",
                    action = "store_true")
parser.add_argument("-l", "--light", help = "lighter plot",
                    action = "store_true")
parser.add_argument("-e", "--end_filename", help = "add character string after "
                    "extremum, outermost contour and max_speed_contour",
                    default = "")
args = parser.parse_args()

reader_extr = shapefile.Reader("extremum" + args.end_filename)
reader_outer = shapefile.Reader("outermost_contour" + args.end_filename)

try:
    reader_m_s = shapefile.Reader("max_speed_contour" + args.end_filename)
except shapefile.ShapefileException:
    print("Shapefile max_speed_contour_1 not found. "
          "Max-speed contours will not be plotted.")
    w = shapefile.Writer(shapeType = shapefile.NULL)
    m_s_iterShapes = itertools.repeat(w)
else:
    m_s_iterShapes = reader_m_s.iterShapes()

with netCDF4.Dataset("h.nc") as f:
    longitude = f.variables["lon"][:]
    latitude = f.variables["lat"][:]
    
if args.window:
    l = input("llcrnrlon, llcrnrlat, urcrnrlon, urcrnrlat = ? ")\
                                                            .split(sep = ",")
    llcrnrlon, llcrnrlat, urcrnrlon, urcrnrlat = [float(v) for v in l]

    if urcrnrlon - llcrnrlon > 360:
        sys.exit("bad values of urcrnrlon and llcrnrlon")

    longitude += np.ceil((llcrnrlon - longitude) / 360) * 360
    # (in [llcrnrlon, llcrnrlon + 2 pi[)

    lon_mask = longitude <= urcrnrlon
    lat_mask = np.logical_and(latitude >= llcrnrlat, latitude <= urcrnrlat)
else:
    lon_mask = np.ones(len(longitude), dtype = bool)
    lat_mask = np.ones(len(latitude), dtype = bool)

plt.figure()
src_crs = ccrs.PlateCarree()
projection = ccrs.PlateCarree()
##projection = ccrs.NorthPolarStereo()
ax = plt.axes(projection = projection)

if args.grid:
    lon_2d, lat_2d = np.meshgrid(longitude[lon_mask], latitude[lat_mask])
    ax.plot(lon_2d.reshape(-1), lat_2d.reshape(-1), transform = src_crs,
            marker = "+", color = "gray", linestyle = "None")

for shape_rec_extr, shape_outer, shape_m_s \
    in zip(reader_extr, reader_outer.iterShapes(), m_s_iterShapes):
    points = shape_rec_extr.shape.points[0]

    if args.window:
        points[0] += math.ceil((llcrnrlon - points[0]) / 360) * 360
        # (in [llcrnrlon, llcrnrlon + 2 pi[)

    if not args.window or (points[0] <= urcrnrlon 
                          and llcrnrlat <= points[1] <= urcrnrlat):
        if shape_rec_extr.record.cyclone == 0:
                # Anti-cyclone
            color = "red"
        else:
            color = "blue"

        lines = ax.plot(points[0], points[1], markersize = 10, color = color,
                        fillstyle = "none", transform = src_crs)

        if shape_rec_extr.record.valid == 1:
            if args.light:
                lines[0].set_marker("+")
            else:
                lines[0].set_marker("o")
        elif not args.light:
            # Invalid outermost contour
            lines[0].set_marker("s")

        if not args.light:
            ax.annotate(str(shape_rec_extr.record.eddy_index),
                        projection.transform_point(points[0], points[1],
                                                   src_crs),
                        xytext = (3, 3), textcoords = "offset points")

        if shape_outer.shapeType != shapefile.NULL:
            points = np.array(shape_outer.points)
            lines = ax.plot(points[:, 0], points[:, 1], color = color,
                            transform = src_crs)

            if not args.light:
                if shape_rec_extr.record.valid == 0:
                    # Invalid outermost contour
                    lines[0].set_marker("s")
                    lines[0].set_fillstyle("none")
                elif shape_m_s.shapeType == shapefile.NULL:
                    lines[0].set_marker("x")
                else:
                    lines[0].set_marker("o")

            if shape_m_s.shapeType != shapefile.NULL:
                points = np.array(shape_m_s.points)
                
                if shape_rec_extr.record.cyclone == 0:
                    # Anti-cyclone
                    color = "magenta"
                else:
                    color = "cyan"

                lines = ax.plot(points[:, 0], points[:, 1], color = color,
                                transform = src_crs)
                if not args.light: lines[0].set_marker("o")

if args.velocity:
    with netCDF4.Dataset("uv.nc") as f:
        quiver_return = ax.quiver(longitude[lon_mask], latitude[lat_mask],
                                  f["u"][0, lat_mask][:, lon_mask],
                                  f["v"][0, lat_mask][:, lon_mask],
                                  scale = args.scale, scale_units = "width",
                                  transform = src_crs)
    plt.quiverkey(quiver_return, 0.9, 0.9, 1, r"1 m s$^{-1}$",
                  coordinates = "figure")

ax.gridlines(draw_labels = True)
ax.coastlines()

plt.show()
